'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = cons;

var _sequence = require('./sequence');

var _sequence2 = _interopRequireDefault(_sequence);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 集合の先頭に要素を追加し、イテレーターを返します。
 *
 * @param {*} x - 追加する要素
 * @param {*[] | iterator | null} coll - 集合
 *
 * @return {iterator} 先頭に要素が追加された集合のイテレーター
 *
 * @example
 * // 無限に続くフィボナッチ数列
 * functon getFibonacciNumbers(x, y) {
 *   return cons(x, lazySeq(getFibonacciNumbers(y, x + y)));
 * }
 */
function* cons(x, coll) {
  yield x;

  for (const y of (0, _sequence2.default)(coll)) {
    yield y;
  }
}