'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = cycle;

var _seq = require('./seq');

var _seq2 = _interopRequireDefault(_seq);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 集合を繰り返した集合のイテレーターを返します。
 *
 * @param {*[] | iterator | null} coll - 集合
 *
 * @return {iterator} 集合を繰り返した集合のイテレーター
 *
 * @example
 * cycle([1, 2, 3]);  // [1, 2, 3, 1, 2, 3, ...]のイテレーター
 */
function* cycle(coll) {
  const iter = (0, _seq2.default)(coll);
  if (!iter) {
    return;
  }

  const xs = [];
  for (const x of iter) {
    xs.push(x);

    yield x;
  }

  for (;;) {
    for (const x of xs) {
      yield x;
    }
  }
}