'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = dropLast;

var _sequence = require('./sequence');

var _sequence2 = _interopRequireDefault(_sequence);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 最後のn個の要素を削除したイテレーターを返します。
 *
 * @param {*[]} args - [n, coll]もしくは[coll]。collのみが指定された場合、nは1になります。
 *
 * @return {iterator} 最後のn個の要素が削除された集合のイテレーター
 *
 * @example
 * dropLast(2, range(5));  // [0, 1, 2]のイテレーター
 * dropLast(   range(5));  // [0, 1, 2, 3]のイテレーター
 */
function* dropLast(...args) {
  const [n, coll] = (() => {
    switch (args.length) {
      case 0:
        throw 'Invalid arguments';

      case 1:
        return [1, args[0]];

      default:
        return args;
    }
  })();

  const xs = [];

  for (const x of (0, _sequence2.default)(coll)) {
    xs.push(x);

    if (xs.length > n) {
      yield xs[0];

      xs.shift();
    }
  }
}