'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = dropWhile;

var _sequence = require('./sequence');

var _sequence2 = _interopRequireDefault(_sequence);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 先頭から続く条件を満たす要素を削除したイテレーターを返します。
 *
 * @param {function} pred - 削除する条件
 * @param {*[] | iterator | null} coll - 集合
 *
 * @return 先頭から続く条件を満たす要素が削除されたイテレーター
 *
 * @example
 * dropWhile(x => x < 2, [0, 1, 2, 3, 2, 1, 0]);  // [2, 3, 2, 1, 0]のイテレーター
 */
function* dropWhile(pred, coll) {
  let b = false;

  for (const x of (0, _sequence2.default)(coll)) {
    if (!b) {
      if (pred(x)) {
        continue;
      }

      b = true;
    }

    yield x;
  }
}