'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = everyPred;

var _every = require('./every');

var _every2 = _interopRequireDefault(_every);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 引数で指定されるすべての条件を満たすかを確認する関数を生成します。
 *
 * @param {function[]} fs - 関数
 *
 * @return {function} 生成された関数
 *
 * @example
 * const isFizz = x => x % 3 === 0;
 * const isBuzz = x => x % 5 === 0;
 * const isFizzBuzz = everyPred(isFizz, isBuzz);
 * isFizzBuzz(15);  // true
 */
function everyPred(...fs) {
  return (...args) => (0, _every2.default)(f => f(...args), fs);
}