'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = filter;

var _sequence = require('./sequence');

var _sequence2 = _interopRequireDefault(_sequence);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 集合から条件を満たす要素を抽出し、その要素の集合のイテレーターを返します。
 *
 * @param {function} pred - 条件
 * @param {*[] | iterator | null} coll - 集合
 *
 * @return {iterator} 抽出された要素の集合のイテレーター
 *
 * @example
 * filter(x => x % 2 === 0, iterate(inc, 1));  // [2, 4, 6, 8, ...]のイテレーター
 */
function* filter(pred, coll) {
  for (const x of (0, _sequence2.default)(coll)) {
    if (pred(x)) {
      yield x;
    }
  }
}