'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = interpose;

var _seq = require('./seq');

var _seq2 = _interopRequireDefault(_seq);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 集合の要素の間にセパレーターを挟み込んだ集合のイテレーターを返します。
 *
 * @param {*} sep - セパレーター
 * @param {*[] | iterator | null} coll - 集合
 *
 * @return {iterator} 集合の要素の間にセパレーターが挟み込まれた集合のイテレーター
 *
 * @example
 * interpose(0, range(1, 5));  // [1, 0, 2, 0, 3, 0, 4]のイテレーター
 */
function* interpose(sep, coll) {
  const iter = (0, _seq2.default)(coll);
  if (!iter) {
    return;
  }

  yield iter.next().value;

  for (const x of iter) {
    yield sep;
    yield x;
  }
}