'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = juxt;

var _isSequencial = require('./isSequencial');

var _isSequencial2 = _interopRequireDefault(_isSequencial);

var _map = require('./map');

var _map2 = _interopRequireDefault(_map);

var _tee = require('./tee');

var _tee2 = _interopRequireDefault(_tee);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 引数で指定した関数群の戻り値の集合へのイテレーターを返す関数を生成します。
 *
 * @param {function[]} fs - 関数
 *
 * @return {function} fsの関数を実行した結果へのイテレーターを返す関数
 *
 * @example
 * const [x, iter] = juxt(first, next)([1, 2, 3]);
 * // x    == 1
 * // iter == [2, 3]のイテレーター
 */
function juxt(...fs) {
  return function (...args) {
    if (args.length === 1 && (0, _isSequencial2.default)(args[0])) {
      return (0, _map2.default)((f, coll) => f(coll), fs, (0, _tee2.default)(fs.length, args[0]));
    }

    return (0, _map2.default)(f => f(...args), fs);
  };
}