'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = lazySeqFn;

var _comp = require('./comp');

var _comp2 = _interopRequireDefault(_comp);

var _sequence = require('./sequence');

var _sequence2 = _interopRequireDefault(_sequence);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * イテレーターが必要となったときまで、関数の実行を遅延します。lazySeqマクロを使用すれば、関数呼び出しをそのまま記述できます。
 *
 * @param {function} f - イテレーターまたはnullを返す関数
 *
 * @return 関数を遅延実行するイテラブル
 *
 * @example
 * // 無限に続くフィボナッチ数列
 * functon getFibonacciNumbers(x, y) {
 *   return cons(x, lazySeq(getFibonacciNumbers(y, x + y)));
 *   // = return cons(x, lazySeqFn(() => getFibonacciNumbers(y, x + y)));
 * }
 */
function lazySeqFn(f) {
  return { [Symbol.iterator]: (0, _comp2.default)(_sequence2.default, f) };
}