'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = maxKey;

var _reduce = require('./reduce');

var _reduce2 = _interopRequireDefault(_reduce);

var _rest = require('./rest');

var _rest2 = _interopRequireDefault(_rest);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * k(x)が最大のxを返します。
 *
 * @param {funciton} k - 比較対象のキーを取得する関数
 * @param {*[]} xs - 値の集合
 *
 * @return {*} k(x)が最大のx
 *
 * @example
 * maxKey(e => e.getSalary(), a, b, c);  // 最も給料が高い社員
 */
function maxKey(k, ...xs) {
  if (xs.length === 0) {
    throw 'Invalid arguments';
  }

  return (0, _reduce2.default)((acc, x) => {
    if (k(x) > acc[0]) {
      return [k(x), x];
    }

    return acc;
  }, [k(xs[0]), xs[0]], (0, _rest2.default)(xs))[1];
}