'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = partitionBy;

var _seq = require('./seq');

var _seq2 = _interopRequireDefault(_seq);

var _sequence = require('./sequence');

var _sequence2 = _interopRequireDefault(_sequence);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 指定した関数の戻り値が変更になるたびに、集合を分割します。
 *
 * @param {function} f - 関数
 * @param {*[] | iterator | null} coll - 集合
 *
 * @return {iterator} 分割された集合のイテレーター
 *
 * @example
 * partitionBy(x => x % 2 === 0, [1, 1, 1, 2, 2, 3, 3, 4]);  // [[1, 1, 1], [2, 2], [3, 3], [4]]のイテレーター
 */
function* partitionBy(f, coll) {
  const iter = (0, _seq2.default)(coll);
  if (!iter) {
    return (0, _sequence2.default)(null);
  }

  const next = iter.next();

  let xs = [next.value];
  let prev = f(next.value);

  for (const x of iter) {
    const r = f(x);
    if (r != prev) {
      yield xs;

      xs = [];
      prev = r;
    }

    xs.push(x);
  }

  yield xs;
}