'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.reducedSymbol = undefined;
exports.default = reduce;

var _cons = require('./cons');

var _cons2 = _interopRequireDefault(_cons);

var _seq = require('./seq');

var _seq2 = _interopRequireDefault(_seq);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * reduced向けのシンボルです。使用しないでください。
 */
const reducedSymbol = exports.reducedSymbol = Symbol('reduced');

/**
 * 畳み込みます。f(f(f(init, coll[0]), coll[1]), coll[2])...の結果を返します。
 *
 * @param {*[]} args - [f, init, coll]か[f, coll]のどちらか。
 *
 * @return {*} 畳み込まれた値
 *
 * @example
 * reduce((acc, x) => acc + x, xs);  // xsの合計
 */
function reduce(...args) {
  const [f, coll] = (() => {
    switch (args.length) {
      case 0:
        throw 'Invalid arguments';

      case 1:
        throw 'Invalid arguments';

      case 2:
        return args;

      default:
        return [args[0], (0, _cons2.default)(args[1], args[2])];
    }
  })();

  const iter = (0, _seq2.default)(coll);
  if (!iter) {
    return null;
  }

  let acc = iter.next().value;

  for (const x of iter) {
    acc = f(acc, x);

    if (acc && acc.hasOwnProperty(reducedSymbol)) {
      acc = acc[reducedSymbol];
      break;
    }
  }

  return acc;
}