'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = reductions;

var _cons = require('./cons');

var _cons2 = _interopRequireDefault(_cons);

var _seq = require('./seq');

var _seq2 = _interopRequireDefault(_seq);

var _reduce = require('./reduce');

var _reduce2 = _interopRequireDefault(_reduce);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * reduceの途中経過も含めて、イテレーターで返します。
 *
 * @param {*[]} args - [f, init, coll]か[f, coll]のどちらか。
 *
 * @return {iterator} 途中経過も含めたreduceの結果のイテレーター
 *
 * @example
 * reduce((acc, x) => acc + x, [1, 2, 3]);  // [1, 3, 6]のイテレーター
 */
function* reductions(...args) {
  const [f, coll] = (() => {
    switch (args.length) {
      case 0:
        throw 'Invalid arguments';

      case 1:
        throw 'Invalid arguments';

      case 2:
        return args;

      default:
        return [args[0], (0, _cons2.default)(args[1], args[2])];
    }
  })();

  const iter = (0, _seq2.default)(coll);
  if (!iter) {
    return;
  }

  let acc = iter.next().value;

  for (const x of iter) {
    yield acc;

    acc = f(acc, x);

    if (acc && acc.hasOwnProperty(_reduce2.default)) {
      acc = acc[_reduce2.default];
      break;
    }
  }

  yield acc;
}